from osv import osv
from osv import fields
import time

#from carpeta import archivo
#archivo.clase
class openacademy_course(osv.osv):
    _name = 'openacademy.course'
    #_table = 'opea_cou'
    #_rec_name = 'description'#por default 'name'
    
    _columns = {
        'name': fields.char('Course Title', size=128, required=True),
        'description': fields.text('Description'),
        'responsible_id': fields.many2one('res.users', string='Responsible', ondelete='set null'),#default is set_null. Too use cascade
        'session_ids': fields.one2many('openacademy.session', 'course_id', string='Sessions'),
    }
    
    _sql_constraints = [
        ('unique_name', 'unique(name)', 'Course Title must be unique'),
    ]
    
    def _check_description(self, cr, uid, ids, context=None):
        course_obj = self.pool.get('openacademy.course')
        courses = course_obj.browse(cr, uid, ids, context=context)#lista de objetos instanciados con los "ids"
        for course in courses:
            if course.name == course.description:
                return False#Paso algo inesperado
        return True#Todo perfecto
    
    _constraints = [
        (_check_description, 'Please use a different description and name', ['name', 'description'])
    ]
    
    def copy(self, cr, uid, id, default=None, context=None):
        if default is None:
            default = {}
        course_obj = self.pool.get('openacademy.course')
        course = course_obj.browse(cr, uid, id, context=context)
        new_name = course.name + ' (copy)'
        course_ids = course_obj.search(cr, uid, [('name','like',new_name)], context=context)
        if len(course_ids) > 0:
            new_name = '%s (copy %s)'%( new_name, len(course_ids)+1)
        default.update({
            'name': new_name,
        })
        return super(openacademy_course, self).copy(cr, uid, id, default, context=context)
openacademy_course()

class openacademy_session(osv.osv):
    _name = 'openacademy.session'
    
    def _get_remaining_seats_percent(self, seats, attendee_list):
        #print "seats",seats
        #print "len attendee_list", len(attendee_list)
        return seats and ((1-len(attendee_list)/(seats*1.0))*100.0) or 0
    
    def _remaining_seats_percent(self, cr, uid, ids, field, arg, context=None):
        sessions = self.pool.get('openacademy.session').browse(cr, uid, ids, context=context)
        result = {}
        for session in sessions:
            #print "session.id",session.id
            #print "session.name",session.name
            #print "session.instructor_id.name",session.instructor_id.name
            #print "session.instructor_id.company_id.name",session.instructor_id.company_id.name
            #print "session.attendee_ids",session.attendee_ids
            #for attendee_id in session.attendee_ids:
                #print "attendee_id.partner_id.name",attendee_id.partner_id.name
            result[session.id] = self._get_remaining_seats_percent(session.seats, session.attendee_ids)
        #result {id: value}
        return result
    
    _columns = {
        'name': fields.char('Session Title', size=128, required=True),
        'start_date': fields.datetime('Start Date'),
        'duration': fields.float('Duration', help='Duration in days'),
        'seats': fields.integer('Seats Number'),
        'course_id': fields.many2one('openacademy.course', string="Course", required=True, ondelete='cascade'),
        'instructor_id': fields.many2one('res.partner', 'Instructor', domain=['|', ('instructor','=',True), ('category_id.name','in',['Teacher Level 1', 'Teacher Level 2'])] ),
        'attendee_ids': fields.one2many('openacademy.attendee', 'session_id', 'Attendees'),
        'remaining_seats_percent': fields.function(_remaining_seats_percent,
            method=True, type='float', string="Remaining Seats"),
        'active': fields.boolean('Active'),
        'state': fields.selection([
                ('draft', 'Draft'),
                ('confirm', 'Confirm'),
                ('done', 'Done'),
            ], 'State', readonly=True, required=True),
    }
    
#    def _get_start_date(self, cr, uid, context=None):
#        return time.strftime('%Y-%m-%d %H:%M:%S')
    
    _defaults = {
        'active': True,
        'start_date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),#_get_start_date,
        'state': 'draft',
    }
    
    def onchange_remaining_seats(self, cr, uid, ids, seats, attendee_ids):
        res = {}
        remaining_seats_percent = self._get_remaining_seats_percent(seats, attendee_ids)
        res.update({
            'value': {
                'remaining_seats_percent': remaining_seats_percent,
            }
        })
        if seats < 0:
        #if remaining_seats_percent < 0:
            res.update({
                'warning': {
                    'title': 'Warning',
                    'message': 'You cannot have negative seats'
                }
            })
        return res#dicc = {
                   # 'value': {'field1': 'valor1', 'field2': 'valor2'}
                    # 'warning': 'title': 'TITLE', 'message': 'MESSAGE'}
        
    def action_confirm(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'confirm'}, context=context)
        return True
        
    def action_done(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'done'}, context=context)
        return True
openacademy_session()

class openacademy_attendee(osv.osv):
    _name = 'openacademy.attendee'
    _rec_name = 'partner_id'
    
    _columns = {
        'partner_id': fields.many2one('res.partner', 'Attendee', required=True),
        'session_id': fields.many2one('openacademy.session', 'Session', required=True, ondelete='cascade'),
    }
openacademy_attendee()
